"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadFunctionsConfig = loadFunctionsConfig;
exports.appendRoutesToFunctionsConfig = appendRoutesToFunctionsConfig;
const ajv_1 = __importDefault(require("ajv"));
const fs_1 = require("fs");
const path = __importStar(require("path"));
let cachedSchema;
function lazyLoadFunctionsJsonSchema() {
    if (cachedSchema) {
        return cachedSchema;
    }
    const config = (0, fs_1.readFileSync)(path.join(__dirname, '../functions-schema.json'), 'utf8');
    cachedSchema = JSON.parse(config);
    return cachedSchema;
}
/**
 * 加载多函数配置
 * @param configFile 配置文件路径
 * @returns 多函数配置
 */
function loadFunctionsConfig(configFile) {
    const config = (0, fs_1.readFileSync)(configFile, 'utf8');
    const parsed = JSON.parse(config);
    const ajv = new ajv_1.default({ strictTuples: false });
    const valid = ajv.validate(lazyLoadFunctionsJsonSchema(), parsed);
    if (!valid) {
        throw new Error(`Invalid functions config: ${ajv.errorsText(ajv.errors)}`);
    }
    if (!parsed.functions) {
        parsed.functions = [];
    }
    if (!parsed.routes) {
        parsed.routes = [];
    }
    appendRoutesToFunctionsConfig(parsed);
    return parsed;
}
function appendRoutesToFunctionsConfig(multiFnConfig) {
    const routeMaps = new Map();
    // 这里包含 routes 重复性校验
    if (Array.isArray(multiFnConfig.routes)) {
        for (const route of multiFnConfig.routes) {
            if (route.path) {
                if (routeMaps.has(route.path)) {
                    throw new Error(`Route path '${route.path}' is duplicated, please check your functions config`);
                }
                routeMaps.set(route.path, route.functionName);
            }
        }
    }
    if (Array.isArray(multiFnConfig.functions)) {
        for (const fn of multiFnConfig.functions) {
            if (fn.triggerPath) {
                if (routeMaps.has(fn.triggerPath)) {
                    const existsTriggerFnName = routeMaps.get(fn.triggerPath);
                    if (existsTriggerFnName === fn.name) {
                        // eslint-disable-next-line max-len
                        // console.warn(`Route '${fn.triggerPath}'-> '${fn.name}' already exists, ignore.`)
                        continue;
                    }
                    else {
                        // eslint-disable-next-line max-len
                        throw new Error(`Route '${fn.triggerPath}->${fn.name}' conflict with '${fn.triggerPath}->${existsTriggerFnName}', please check your functions config`);
                    }
                }
                multiFnConfig.routes.push({ functionName: fn.name, path: fn.triggerPath });
                routeMaps.set(fn.triggerPath, fn.name);
            }
        }
    }
}
//# sourceMappingURL=functions-config.js.map